-- Recovery options demo

IF DATABASEPROPERTYEX (N'Company', N'Version') > 0
BEGIN
	ALTER DATABASE [Company] SET SINGLE_USER
		WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [Company];
END
GO

-- Create the database
CREATE DATABASE [Company] ON PRIMARY (
    NAME = N'Company',
    FILENAME = N'D:\SQLskills\Company.mdf')
LOG ON (
    NAME = N'Company_log',
    FILENAME = N'D:\SQLskills\Company_log.ldf');
GO

-- Create a table
CREATE TABLE [Company]..[TestTable] (
	c1 INT IDENTITY,
	c2 VARCHAR (100));
GO

INSERT INTO [Company]..[TestTable]
	VALUES ('Initial data: transaction 1');
GO

-- And take a full backup
BACKUP DATABASE [Company] TO
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH INIT;
GO

-- Now add some more data and a backup
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 2');
GO
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 3');
GO

BACKUP LOG [Company] TO
	DISK = N'C:\SQLskills\Company_Log1.bck'
WITH INIT;
GO

-- And more data
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 4');
GO
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 5');
GO
-- Now check the time and save it
SELECT GETDATE ();
GO

-- Saved time: 

-- And add some more data
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 6');
GO
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 7');
GO

-- And take another backup
BACKUP LOG [Company] TO
	DISK = N'C:\SQLskills\Company_Log2.bck'
WITH INIT;
GO

-- Simulate disaster
DROP DATABASE [Company];
GO

-- Restore the full backup
RESTORE DATABASE [Company] FROM
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH REPLACE;
GO

-- And the log backups
RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log1.bck';

RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log2.bck';
GO

-- WITH RECOVERY is the DEFAULT!
-- We need to start again...
RESTORE DATABASE [Company] FROM
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH NORECOVERY, REPLACE;
GO

RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log1.bck'
WITH NORECOVERY;

RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log2.bck'
WITH NORECOVERY;
GO

-- And then finalize recovery
RESTORE DATABASE [Company] WITH RECOVERY;
GO

SELECT * from [Company]..[TestTable];
GO

-- What if we want to examine things
-- between restores?
RESTORE DATABASE [Company] FROM
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH REPLACE,
	STANDBY = N'C:\SQLskills\standbyfile.dat';
GO

-- We can see what's in there...
SELECT * from [Company]..[TestTable];
GO

-- Can we update anything?
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 6');
GO

-- Let's do the next log file.
RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log1.bck'
WITH STANDBY = 'C:\SQLskills\standbyfile.dat';
GO

-- Now what's there?
SELECT * from [Company]..[TestTable];
GO

-- Now restore the rest and bring the
-- database online
RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log2.bck'
WITH NORECOVERY;
GO

RESTORE DATABASE [Company] WITH RECOVERY;
GO

SELECT * from [Company]..[TestTable];
GO

-- What if we want to stop at a specific point?
RESTORE DATABASE [Company] FROM
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH NORECOVERY, REPLACE,
	STOPAT = 'XXXX';
GO

RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log1.bck'
WITH STOPAT = 'XXXX',
	STANDBY = 'C:\SQLskills\standbyfile.dat';

-- Are we there yet?
SELECT * from [Company]..[TestTable];
GO

RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_Log2.bck'
WITH STOPAT = 'XXXX',
	STANDBY = 'C:\SQLskills\standbyfile.dat';
GO

-- Is this the right point?
SELECT * from [Company]..[TestTable];
GO

-- Finalize recovery
RESTORE DATABASE [Company] WITH RECOVERY;
GO
SELECT * from [Company]..[TestTable];
GO

-- Take another backup so we don't need
-- to go through all that again
BACKUP DATABASE [Company] TO
	DISK = N'C:\SQLskills\Company_NewFull.bck'
WITH INIT;
GO

