-- Tail of the log backups

IF DATABASEPROPERTYEX (N'Company', N'Version') > 0
BEGIN
	ALTER DATABASE [Company] SET SINGLE_USER
		WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [Company];
END
GO

-- Create the database
CREATE DATABASE [Company] ON PRIMARY (
    NAME = N'Company',
    FILENAME = N'D:\SQLskills\Company.mdf')
LOG ON (
    NAME = N'Company_log',
    FILENAME = N'D:\SQLskills\Company_log.ldf');
GO

-- Create a table
CREATE TABLE [Company]..[TestTable] (
	[c1] INT IDENTITY,
	[c2] VARCHAR (100));
GO

INSERT INTO [Company]..[TestTable]
	VALUES ('Initial data: transaction 1');
GO

-- And take a full backup
BACKUP DATABASE [Company] TO
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH INIT;
GO

-- Now add some more data
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 2');
GO
INSERT INTO [Company]..[TestTable]
	VALUES ('Transaction 3');
GO

-- Simulate a crash
ALTER DATABASE [Company] SET OFFLINE;
GO

-- Trash the data file and bring the database back online

ALTER DATABASE [Company] SET ONLINE;
GO

USE [Company];
GO

-- The backup doesn't have the most recent
-- transactions - if we restore it we'll
-- lose them.

-- Take a log backup?
BACKUP LOG [Company] TO
	DISK = N'C:\SQLskills\Company_tail.bck'
WITH INIT;
GO

-- Use the special syntax!
BACKUP LOG [Company] TO
	DISK = N'C:\SQLskills\Company_tail.bck'
WITH INIT, NO_TRUNCATE;
GO

-- Now restore
RESTORE DATABASE [Company] FROM
	DISK = N'C:\SQLskills\Company_Full.bck'
WITH REPLACE, NORECOVERY;
GO

RESTORE LOG [Company] FROM
	DISK = N'C:\SQLskills\Company_tail.bck';
GO

-- Is everything there?
SELECT * FROM [Company]..[TestTable];
GO

